@@ -71,7 +71,6 @@ from ..util import (
 from ..util.i18n import _
 from . import (
     branchop,
-    csinfo,
     cmdcore,
     cmdui,
     hgrcutil,
@@ -80,10 +79,16 @@ from . import (
     qtlib,
     revpanel,
     status,
+    topic,
     thgrepo,
 )
 from .messageentry import MessageEntry
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Text,
+    )
+
 if os.name == 'nt':
     from ..util import bugtraq
     _hasbugtraq = True
@@ -252,6 +257,15 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.branchbutton.triggered.connect(self.branchOp)
         self.branchop = None
 
+        if hglib.hastopicext(repo):
+            self.topicbutton = tbar.addAction(_('Topic: '))
+            font = self.topicbutton.font()
+            font.setBold(True)
+            self.topicbutton.setFont(font)
+            self.topicbutton.triggered.connect(self.topicOp)
+        else:
+            self.topicbutton = None
+
         self.recentMessagesButton = QToolButton(
             text=_('Copy message'),
             popupMode=QToolButton.InstantPopup,
@@ -773,6 +787,14 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             title = _('New Branch: ') + self.branchop
         self.branchbutton.setText(title)
 
+        if self.topicbutton:
+            topic = getattr(self.repo, 'currenttopic')
+            if topic:
+                topic = hglib.tounicode(topic)
+            else:
+                topic = u"None"
+            self.topicbutton.setText(_('Topic: ') + topic)
+
         # Update options label, showing only whitelisted options.
         opts = commitopts2str(self.opts)
         self.optionslabelfmt = _('<b>Selected Options:</b> %s')
@@ -820,6 +842,14 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.msgte.setFocus()
             self.refresh()
 
+    def topicOp(self):
+        d = topic.TopicDialog(self._repoagent, self.branchop, self)
+        d.setWindowFlags(Qt.Sheet)
+        d.setWindowModality(Qt.WindowModal)
+        d.exec_()
+        self.msgte.setFocus()
+        self.refresh()
+
     def canUndo(self):
         'Returns undo description or None if not valid'
         desc, oldlen = hglib.readundodesc(self.repo)
@@ -929,7 +959,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.updateRecentMessages()
 
     def addUsernameToHistory(self, user):
-        user = hglib.tounicode(user)
+        # type: (Text) -> None
         if user in self.userhist:
             self.userhist.remove(user)
         self.userhist.insert(0, user)
@@ -1428,7 +1458,8 @@ class DetailsDialog(QDialog):
     def accept(self):
         outopts = {}
         if self.datecb.isChecked():
-            date = hglib.fromunicode(self.datele.text())
+            udate = self.datele.text()
+            date = hglib.fromunicode(udate)
             try:
                 dateutil.parsedate(date)
             except error.ParseError as e:
@@ -1439,12 +1470,12 @@ class DetailsDialog(QDialog):
                     err = hglib.tounicode(str(e))
                 qtlib.WarningMsgBox(_('Invalid date format'), err, parent=self)
                 return
-            outopts['date'] = date
+            outopts['date'] = udate
         else:
             outopts['date'] = ''
 
         if self.usercb.isChecked():
-            user = hglib.fromunicode(self.usercombo.currentText())
+            user = self.usercombo.currentText()
         else:
             user = ''
         outopts['user'] = user
@@ -1462,13 +1493,13 @@ class DetailsDialog(QDialog):
                 return
 
         if self.pushaftercb.isChecked():
-            remote = hglib.fromunicode(self.pushafterle.text())
+            remote = self.pushafterle.text()
             outopts['pushafter'] = remote
         else:
             outopts['pushafter'] = ''
 
         if self.autoinccb.isChecked():
-            outopts['autoinc'] = hglib.fromunicode(self.autoincle.text())
+            outopts['autoinc'] = self.autoincle.text()
         else:
             outopts['autoinc'] = ''
 
