@@ -35,6 +35,7 @@ from .qtcore import (
     pyqtSlot,
 )
 from .qtgui import (
+    QAction,
     QApplication,
     QComboBox,
     QCommonStyle,
@@ -64,6 +65,7 @@ from .qtgui import (
 
 from mercurial import (
     color,
+    encoding,
     extensions,
     pycompat,
     util,
@@ -87,12 +89,22 @@ try:
 except ImportError:
     openflags = 0
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        List,
+        Text,
+        Tuple,
+        Optional,
+    )
+
 if pycompat.ispy3:
     from html import escape as htmlescape
 else:
     import cgi
     def htmlescape(s, quote=True):
-        return cgi.escape(s, quote)
+        return cgi.escape(s, quote)  # pytype: disable=module-attr
 
 # largest allowed size for widget, defined in <src/gui/kernel/qwidget.h>
 QWIDGETSIZE_MAX = (1 << 24) - 1
@@ -133,14 +145,12 @@ def openhelpcontents(url):
 def openlocalurl(path):
     '''open the given path with the default application
 
-    takes str, unicode or QString as argument
+    takes bytes or unicode as argument
     returns True if open was successfull
     '''
 
-    if isinstance(path, str):
+    if isinstance(path, bytes):
         path = hglib.tounicode(path)
-    else:
-        path = pycompat.unicode(path)
     if os.name == 'nt' and path.startswith('\\\\'):
         # network share, special handling because of qt bug 13359
         # see https://bugreports.qt.io/browse/QTBUG-13359
@@ -151,12 +161,12 @@ def openlocalurl(path):
     return QDesktopServices.openUrl(qurl)
 
 def editfiles(repo, files, lineno=None, search=None, parent=None):
+    # type: (Any, List[bytes], Optional[int], Optional[bytes], Optional[QWidget]) -> None
     if len(files) == 1:
         # if editing a single file, open in cwd context of that file
         filename = files[0].strip()
         if not filename:
             return
-        files = [filename]
         path = repo.wjoin(filename)
         cwd = os.path.dirname(path)
         files = [os.path.basename(path)]
@@ -166,16 +176,15 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
 
     toolpath, args, argsln, argssearch = editor.detecteditor(repo, files)
     if os.path.basename(toolpath) in (b'vi', b'vim', b'hgeditor'):
-        res = QMessageBox.critical(parent,
-                    _('No visual editor configured'),
-                    _('Please configure a visual editor.'))
+        QMessageBox.critical(parent, _('No visual editor configured'),
+                             _('Please configure a visual editor.'))
         from tortoisehg.hgqt.settings import SettingsDialog
         dlg = SettingsDialog(False, focus='tortoisehg.editor')
         dlg.exec_()
         return
 
     files = [procutil.shellquote(util.localpath(f)) for f in files]
-    assert len(files) == 1 or lineno == None, (files, lineno)
+    assert len(files) == 1 or lineno is None, (files, lineno)
 
     cmdline = None
     if search:
@@ -189,7 +198,7 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
             cmdline = cmdline.replace(b'$LINENUM', b'%d' % lineno)
         elif args:
             cmdline = b' '.join([toolpath, args])
-    elif lineno:
+    elif lineno is not None:
         if argsln:
             cmdline = b' '.join([toolpath, argsln])
             cmdline = cmdline.replace(b'$LINENUM', b'%d' % lineno)
@@ -227,10 +236,10 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
                 expanded.append(phrase)
             expanded.append(toolpath[pos:])
             cmdline = b' '.join(expanded + files)
-        except ValueError as e:
+        except ValueError:
             # '[' or ']' not found
             pass
-        except TypeError as e:
+        except TypeError:
             # variable expansion failed
             pass
 
@@ -377,7 +386,7 @@ def isDarkTheme(palette=None):
 # effect programatically.
 
 # TODO: update ui._styles instead of color._defaultstyles
-_styles = pycompat.rapply(pycompat.sysstr, color._defaultstyles)
+_styles = pycompat.rapply(pycompat.sysstr, color._defaultstyles)  # type: Dict[str, str]
 
 _effects = {
     'bold': 'font-weight: bold',
@@ -386,21 +395,25 @@ _effects = {
 }
 
 _thgstyles = {
-   # Styles defined by TortoiseHg
-   'log.branch': 'black #aaffaa_background',
-   'log.patch': 'black #aaddff_background',
-   'log.unapplied_patch': 'black #dddddd_background',
-   'log.tag': 'black #ffffaa_background',
-   'log.bookmark': 'blue #ffffaa_background',
-   'log.curbookmark': 'black #ffdd77_background',
-   'log.modified': 'black #ffddaa_background',
-   'log.added': 'black #aaffaa_background',
-   'log.removed': 'black #ffcccc_background',
-   'log.warning': 'black #ffcccc_background',
-   'status.deleted': 'red bold',
-   'ui.error': 'red bold #ffcccc_background',
-   'ui.warning': 'black bold #ffffaa_background',
-   'control': 'black bold #dddddd_background',
+    # Styles defined by TortoiseHg
+    'log.branch': 'black #aaffaa_background',
+    'log.patch': 'black #aaddff_background',
+    'log.unapplied_patch': 'black #dddddd_background',
+    'log.tag': 'black #ffffaa_background',
+    'log.bookmark': 'blue #ffffaa_background',
+    'log.curbookmark': 'black #ffdd77_background',
+    'log.modified': 'black #ffddaa_background',
+    'log.added': 'black #aaffaa_background',
+    'log.removed': 'black #ffcccc_background',
+    'log.warning': 'black #ffcccc_background',
+    'status.deleted': 'red bold',
+    'ui.error': 'red bold #ffcccc_background',
+    'ui.warning': 'black bold #ffffaa_background',
+    'control': 'black bold #dddddd_background',
+
+    # Topic related styles
+    'log.topic': 'black bold #2ecc71_background',
+    'topic.active': 'black bold #2ecc71_background',
 }
 
 thgstylesheet = '* { white-space: pre; font-family: monospace;' \
@@ -417,17 +430,17 @@ def configstyles(ui):
     _styles.update(_thgstyles)
 
     # allow the user to override
-    for status, cfgeffects in ui.configitems(b'color'):
+    for status, cfgeffects in ui.configitems(b'color'):  # type: Tuple[bytes, bytes]
         if b'.' not in status:
             continue
         cfgeffects = ui.configlist(b'color', status)
-        _styles[status] = b' '.join(cfgeffects)
+        _styles[pycompat.sysstr(status)] = pycompat.sysstr(b' '.join(cfgeffects))
 
-    for status, cfgeffects in ui.configitems(b'thg-color'):
+    for status, cfgeffects in ui.configitems(b'thg-color'):  # type: Tuple[bytes, bytes]
         if b'.' not in status:
             continue
         cfgeffects = ui.configlist(b'thg-color', status)
-        _styles[status] = b' '.join(cfgeffects)
+        _styles[pycompat.sysstr(status)] = pycompat.sysstr(b' '.join(cfgeffects))
 
 # See https://doc.qt.io/qt-4.8/richtext-html-subset.html
 # and https://www.w3.org/TR/SVG/types.html#ColorKeywords
@@ -477,10 +490,19 @@ def getbgcoloreffect(labels):
                 return QColor(e[:-11])
     return QColor()
 
+# TortoiseHg uses special names for the properties controlling the appearance of
+# its interface elements.
+#
+# This dict maps internal style names to corresponding CSS property names.
 NAME_MAP = {
-    'fg': 'color', 'bg': 'background-color', 'family': 'font-family',
-    'size': 'font-size', 'weight': 'font-weight', 'space': 'white-space',
-    'style': 'font-style', 'decoration': 'text-decoration',
+    'fg': 'color',
+    'bg': 'background-color',
+    'family': 'font-family',
+    'size': 'font-size',
+    'weight': 'font-weight',
+    'space': 'white-space',
+    'style': 'font-style',
+    'decoration': 'text-decoration',
 }
 
 def markup(msg, **styles):
@@ -723,7 +745,7 @@ def getcheckboxpixmap(state, bgcolor, widget):
 # icons to be rendered unusably small. The workaround for that is to render
 # the icons at double the normal size.
 # TODO: Remove this hack after upgrading to Qt5.
-IS_RETINA = stringutil.parsebool(os.environ.get('THG_RETINA', '0'))
+IS_RETINA = stringutil.parsebool(encoding.environ.get(b'THG_RETINA', b'0'))
 
 def _fixIconSizeForRetinaDisplay(s):
     if IS_RETINA:
@@ -1375,6 +1397,7 @@ class Spacer(QWidget):
         return QSize(self.width, self.height)
 
 def getCurrentUsername(widget, repo, opts=None):
+    # type: (Optional[QWidget], Any, Optional[Dict[Text, Text]]) -> Optional[Text]
     if opts:
         # 1. Override has highest priority
         user = opts.get('user')
@@ -1384,7 +1407,7 @@ def getCurrentUsername(widget, repo, opts=None):
     # 2. Read from repository
     user = hglib.configuredusername(repo.ui)
     if user:
-        return user
+        return hglib.tounicode(user)
 
     # 3. Get a username from the user
     QMessageBox.information(widget, _('Please enter a username'),
@@ -1394,7 +1417,7 @@ def getCurrentUsername(widget, repo, opts=None):
     dlg = SettingsDialog(False, focus='ui.username')
     dlg.exec_()
     repo.invalidateui()
-    return hglib.configuredusername(repo.ui)
+    return hglib.tounicode(hglib.configuredusername(repo.ui))
 
 class _EncodingSafeInputDialog(QInputDialog):
     def accept(self):
@@ -1429,11 +1452,6 @@ def keysequence(o):
     except AttributeError:
         return QKeySequence(o)
 
-def modifiedkeysequence(o, modifier):
-    """Create QKeySequence of modifier key prepended"""
-    origseq = QKeySequence(keysequence(o))
-    return QKeySequence('%s+%s' % (modifier, origseq.toString()))
-
 def newshortcutsforstdkey(key, *args, **kwargs):
     """Create [QShortcut,...] for all key bindings of the given StandardKey"""
     return [QShortcut(keyseq, *args, **kwargs)
@@ -1486,3 +1504,10 @@ def setContextMenuShortcut(action, shortcut):
     action.setShortcut(shortcut)
     if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
         action.setShortcutVisibleInContextMenu(True)
+
+def setContextMenuShortcuts(action, shortcuts):
+    # type: (QAction, List[QKeySequence]) -> None
+    """Set shortcuts for a context menu action, making sure it's visible"""
+    action.setShortcuts(shortcuts)
+    if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
+        action.setShortcutVisibleInContextMenu(True)
