@@ -56,7 +56,7 @@ except ImportError:
     config_nofork = None
 
 shortlicense = b'''
-Copyright (C) 2008-2020 Steve Borho <steve@borho.org> and others.
+Copyright (C) 2008-2021 Steve Borho <steve@borho.org> and others.
 This is free software; see the source for copying conditions.  There is NO
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 '''
@@ -344,16 +344,16 @@ def _runcatch(ui, args):
             ui.flush()
     except error.AmbiguousCommand as inst:
         ui.warn(_("thg: command '%s' is ambiguous:\n    %s\n") %
-                (inst.args[0], b" ".join(inst.args[1])))
+                (inst.prefix, b" ".join(inst.matches)))
     except error.CommandError as inst:
-        if inst.args[0]:
-            ui.warn(_("thg %s: %s\n") % (inst.args[0], inst.args[1]))
-            help_(ui, inst.args[0])
+        if inst.command:
+            ui.warn(_("thg %s: %s\n") % (inst.command, inst.message))
+            help_(ui, inst.command)
         else:
-            ui.warn(_("thg: %s\n") % inst.args[1])
+            ui.warn(_("thg: %s\n") % inst.message)
             help_(ui, b'shortlist')
     except error.UnknownCommand as inst:
-        ui.warn(_("thg: unknown command '%s'\n") % inst.args[0])
+        ui.warn(_("thg: unknown command '%s'\n") % inst.command)
         help_(ui, b'shortlist')
     except error.RepoError as inst:
         ui.warn(_("abort: %s!\n") % inst)
@@ -583,7 +583,7 @@ def annotate(ui, repoagent, *pats, **opts):
 def archive(ui, repoagent, *pats, **opts):
     """archive dialog"""
     from tortoisehg.hgqt import archive as archivemod
-    rev = opts.get('rev')
+    rev = hglib.tounicode(opts.get('rev'))
     return archivemod.createArchiveDialog(repoagent, rev)
 
 @command(b'backout',
@@ -692,7 +692,7 @@ def debugconsole(ui, repoagent, *pats, **opts):
 @command(b'debuglighthg', [], _('thg debuglighthg'))
 def debuglighthg(ui, repoagent, *pats, **opts):
     from tortoisehg.hgqt import repowidget
-    return repowidget.LightRepoWindow(repoagent)
+    return repowidget.LightRepoWindow(qtrun.actionRegistry(), repoagent)
 
 @command(b'debugruncommand', [],
     _('thg debugruncommand -- COMMAND [ARGUMENT]...'))
@@ -831,7 +831,7 @@ def help_(ui, name=None, with_version=False, **opts):
         try:
             aliases, i = cmdutil.findcmd(name, table, False)
         except error.AmbiguousCommand as inst:
-            select = lambda c: c.startswith(inst.args[0])
+            select = lambda c: c.startswith(inst.prefix)
             helplist(_('list of commands:\n\n'), select)
             return
 
@@ -1311,7 +1311,7 @@ def sync(ui, repoagent, url=None, **opts):
 
     repo = repoagent.rawRepo()
     repo.ui.setconfig(b'tortoisehg', b'defaultwidget', b'sync')
-    w = repowidget.LightRepoWindow(repoagent)
+    w = repowidget.LightRepoWindow(qtrun.actionRegistry(), repoagent)
     if url:
         w.setSyncUrl(url)
     return w
@@ -1358,7 +1358,7 @@ def topics(ui, repoagent, *names, **opts):
     if b'topic' not in repo.extensions():
         raise error.Abort(_('Please enable the Topic extension first.'))
     from tortoisehg.hgqt import topic as topicmod
-    rev = scmutil.revsingle(repo, opts.get('rev')).rev()
+    rev = scmutil.revsingle(repo, opts.get('rev'), default=None).rev()
     if len(names) > 1:
         raise error.Abort(_('only one new topic name allowed'))
     dlg = topicmod.TopicDialog(repoagent, rev)
