@@ -58,6 +58,10 @@ from mercurial import (
     util,
 )
 
+from mercurial.utils import (
+    stringutil,
+)
+
 from ..util import (
     editor,
     gpg,
@@ -129,7 +133,7 @@ class SettingsCombo(QComboBox):
         self.clear()
         ucur = hglib.tounicode(self.curvalue)
         if self.opts.get('defer') and not self.loaded:
-            if self.curvalue == None: # unspecified
+            if self.curvalue is None:  # unspecified
                 self.addItem(_unspecstr)
             else:
                 self.addItem(ucur or '...')
@@ -204,10 +208,16 @@ class BoolRBGroup(QWidget):
 
     ## common APIs for all edit widgets
     def setValue(self, curvalue):
-        self.curvalue = curvalue
-        if curvalue == b'True':
+        # Store parsed value, so a compare of 1 to True isn't flagged as dirty
+        if curvalue:
+            value = pycompat.unicode(stringutil.parsebool(curvalue))
+            self.curvalue = hglib.fromunicode(value)
+        else:
+            self.curvalue = None
+
+        if self.curvalue == b'True':
             self.trueRB.setChecked(True)
-        elif curvalue == b'False':
+        elif self.curvalue == b'False':
             self.falseRB.setChecked(True)
         else:
             self.unspecRB.setChecked(True)
@@ -376,7 +386,7 @@ class SettingsCheckBox(QCheckBox):
         self.setText(opts['label'])
 
     def setValue(self, curvalue):
-        if self.curvalue == None:
+        if self.curvalue is None:
             self.curvalue = curvalue
         self.setChecked(curvalue)
 
@@ -425,9 +435,9 @@ class BugTraqConfigureEntry(QPushButton):
 
     def master_updated(self):
         self.setEnabled(False)
-        if self.master == None:
+        if self.master is None:
             return
-        if self.master.value() == None:
+        if self.master.value() is None:
             return
         if len(self.master.value()) == 0:
             return
@@ -452,7 +462,7 @@ class BugTraqConfigureEntry(QPushButton):
 
     ## common APIs for all edit widgets
     def setValue(self, curvalue):
-        if self.master == None:
+        if self.master is None:
             self.master = self.opts['master']
             self.master.currentIndexChanged.connect(self.master_updated)
         self.master_updated()
@@ -622,7 +632,7 @@ class _fi(object):
         self.visible = visible
 
     def isVisible(self):
-        if self.visible == None:
+        if self.visible is None:
             return True
         else:
             return self.visible()
@@ -1579,14 +1589,14 @@ class SettingsForm(QWidget):
                     val = self.readCPath('extensions.' + fullkey)
                     if val is not None:
                         break
-                if val == None:
+                if val is None:
                     curvalue = False
-                elif len(val) and val[0] == '!':
+                elif val.startswith(b'!'):
                     curvalue = False
                 else:
                     curvalue = True
                 w.setValue(curvalue)
-                if val == None:
+                if val is None:
                     w.opts['cpath'] = 'extensions.' + key
                 else:
                     w.opts['cpath'] = 'extensions.' + fullkey
@@ -1776,7 +1786,7 @@ class SettingsForm(QWidget):
         section, key = pycompat.sysbytes(cpath).split(b'.', 1)
         if newvalue == self.ini.get(section, key):
             return False
-        if newvalue == None:
+        if newvalue is None:
             try:
                 del self.ini[section][key]
             except KeyError:
@@ -1816,7 +1826,6 @@ class SettingsForm(QWidget):
     def applyChangesForExtensions(self):
         assert 'extensions' in self.pages
         emitChanged = False
-        section = 'extensions'
         enabledexts = hglib.enabledextensions()
         for chk in self.pages['extensions'][2]:
             if (not emitChanged) and chk.isDirty():
@@ -1838,7 +1847,6 @@ class SettingsForm(QWidget):
 
     @pyqtSlot()
     def validateextensions(self):
-        section = 'extensions'
         enabledexts = hglib.enabledextensions()
         selectedexts = set(chk.opts['label']
                            for chk in self.pages['extensions'][2]
@@ -1846,7 +1854,7 @@ class SettingsForm(QWidget):
         invalidexts = hglib.validateextensions(selectedexts)
 
         def getinival(cpath):
-            if section not in self.ini:
+            if b'extensions' not in self.ini:
                 return None
             sect, key = pycompat.sysbytes(cpath).split(b'.', 1)
             try:
@@ -1856,7 +1864,7 @@ class SettingsForm(QWidget):
 
         def changable(name, cpath):
             curval = getinival(cpath)
-            if curval not in ('', None):
+            if curval not in (b'', None):
                 # enabled or unspecified, official extensions only
                 return False
             elif name in enabledexts and curval is None:
@@ -1877,6 +1885,11 @@ class SettingsForm(QWidget):
                 sect, key = pycompat.sysbytes(cpath).split(b'.', 1)
                 if hglib.loadui().config(sect, key, None) is not None:
                     chk.setValue(True)
+                    # XXX: setValue(False) was called in refreshPage() prior to
+                    #      this.  SettingsCheckBox only sets its value if the
+                    #      current value is None, but this needs to be forced on
+                    #      because the dirty check looks at the checkbox state
+                    #      and will flag the item as dirty otherwise.
                     chk.curvalue = True
             else:
                 chk.setEnabled(True)
