@@ -16,7 +16,6 @@ import sys
 
 from .qtcore import (
     QSettings,
-    QT_VERSION,
     Qt,
     pyqtSlot,
 )
@@ -52,6 +51,7 @@ from . import (
     qtlib,
     repotab,
     serve,
+    shortcutsettings,
 )
 from .docklog import LogDockWidget
 from .reporegistry import RepoRegistryView
@@ -60,10 +60,11 @@ from .settings import SettingsDialog
 class Workbench(QMainWindow):
     """hg repository viewer/browser application"""
 
-    def __init__(self, ui, config, repomanager):
+    def __init__(self, ui, config, actionregistry, repomanager):
         QMainWindow.__init__(self)
         self.ui = ui
         self._config = config
+        self._actionregistry = actionregistry
         self._repomanager = repomanager
         self._repomanager.configChanged.connect(self._setupUrlComboIfCurrent)
 
@@ -114,16 +115,7 @@ class Workbench(QMainWindow):
                                     self.cloneRepository)
             self.dockMenu.addAction(_('&Open Repository...'),
                                     self.openRepository)
-            if QT_VERSION < 0x50000:
-                from .qtgui import (
-                    qt_mac_set_dock_menu,
-                    qt_mac_set_menubar_icons,
-                )
-                qt_mac_set_dock_menu(self.dockMenu)
-                # On Mac OS X, we do not want icons on menus
-                qt_mac_set_menubar_icons(False)
-            else:
-                self.dockMenu.setAsDockMenu()
+            self.dockMenu.setAsDockMenu()
 
         self._dialogs = qtlib.DialogKeeper(
             lambda self, dlgmeth: dlgmeth(self), parent=self)
@@ -133,7 +125,7 @@ class Workbench(QMainWindow):
         self.resize(desktopgeom.size() * 0.8)
 
         self.repoTabsWidget = tw = repotab.RepoTabWidget(
-            self._config, self._repomanager, self)
+            self._config, self._actionregistry, self._repomanager, self)
         sp = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         sp.setHorizontalStretch(1)
         sp.setVerticalStretch(1)
@@ -180,49 +172,48 @@ class Workbench(QMainWindow):
         self._actionavails = {'repoopen': []}
         self._actionvisibles = {'repoopen': []}
 
-        modifiedkeysequence = qtlib.modifiedkeysequence
         newaction = self._addNewAction
+        newnamed = self._addNewNamedAction
         newseparator = self._addNewSeparator
 
-        newaction(_("New &Workbench"), self.newWorkbench,
-                  shortcut='Shift+Ctrl+W', menu='file', icon='hg-log')
+        newnamed('Workbench.newWorkbench', self.newWorkbench,
+                 menu='file', icon='hg-log')
         newseparator(menu='file')
-        newaction(_("&New Repository..."), self.newRepository,
-                  shortcut='New', menu='file', icon='hg-init')
-        newaction(_("Clon&e Repository..."), self.cloneRepository,
-                  shortcut=modifiedkeysequence('New', modifier='Shift'),
-                  menu='file', icon='hg-clone')
+        newnamed('Workbench.newRepository', self.newRepository,
+                 menu='file', icon='hg-init')
+        newnamed('Workbench.cloneRepository', self.cloneRepository,
+                 menu='file', icon='hg-clone')
         newseparator(menu='file')
-        newaction(_("&Open Repository..."), self.openRepository,
-                  shortcut='Open', menu='file')
-        newaction(_("&Close Repository"), self.closeCurrentRepoTab,
-                  shortcut='Close', enabled='repoopen', menu='file')
+        newnamed('Workbench.openRepository', self.openRepository, menu='file')
+        newnamed('Workbench.closeRepository', self.closeCurrentRepoTab,
+                 enabled='repoopen', menu='file')
         newseparator(menu='file')
         self.menuFile.addActions(self.repoTabsWidget.tabSwitchActions())
         newseparator(menu='file')
-        newaction(_('&Settings'), self.editSettings, icon='thg-userconfig',
-                  shortcut='Preferences', menu='file')
+        newnamed('Workbench.openSettings', self.editSettings,
+                 icon='thg-userconfig', menu='file')
+        newnamed('Workbench.openShortcutSettings',
+                 self._openShortcutSettingsDialog, menu='file')
         newseparator(menu='file')
-        newaction(_("E&xit"), self.close, shortcut='Quit', menu='file')
+        newnamed('Workbench.quit', self.close, menu='file')
 
         a = self.reporegistry.toggleViewAction()
-        a.setText(_('Sh&ow Repository Registry'))
-        a.setShortcut('Ctrl+Shift+O')
         a.setIcon(qtlib.geticon('thg-reporegistry'))
+        self._actionregistry.registerAction('Workbench.showRepoRegistry', a)
         self.docktbar.addAction(a)
         self.menuView.addAction(a)
 
         a = self.mqpatches.toggleViewAction()
-        a.setText(_('Show &Patch Queue'))
         a.setIcon(qtlib.geticon('thg-mq'))
+        self._actionregistry.registerAction('Workbench.showPatchQueue', a)
         self.docktbar.addAction(a)
         self.menuView.addAction(a)
 
-        self._actionShowConsole = a = QAction(_('Show Conso&le'), self)
+        self._actionShowConsole = a = QAction(self)
         a.setCheckable(True)
-        a.setShortcut('Ctrl+L')
         a.setIcon(qtlib.geticon('thg-console'))
         a.triggered.connect(self._setConsoleVisible)
+        self._actionregistry.registerAction('Workbench.showConsole', a)
         self.docktbar.addAction(a)
         self.menuView.addAction(a)
 
@@ -237,8 +228,8 @@ class Workbench(QMainWindow):
         menu.addActions(self.reporegistry.settingActions())
 
         newseparator(menu='view')
-        newaction(_("C&hoose Log Columns..."), self._setHistoryColumns,
-                  enabled='repoopen', menu='view')
+        newnamed('RepoView.setHistoryColumns', self._setHistoryColumns,
+                 enabled='repoopen', menu='view')
         self.actionSaveRepos = \
         newaction(_("Save Open Repositories on E&xit"), checkable=True,
                   menu='view')
@@ -285,50 +276,46 @@ class Workbench(QMainWindow):
 
         newseparator(menu='view')
 
-        a = newaction(_("&Refresh"), self.refresh, icon='view-refresh',
-                      enabled='repoopen', menu='view', toolbar='edit',
-                      tooltip=_('Refresh current repository'))
-        a.setShortcuts(QKeySequence.keyBindings(QKeySequence.Refresh)
-                       + [QKeySequence('Ctrl+F5')])  # Ctrl+ to ignore status
-        newaction(_("Refresh &Task Tab"), self._repofwd('reloadTaskTab'),
-                  enabled='repoopen',
-                  shortcut=modifiedkeysequence('Refresh', modifier='Shift'),
-                  tooltip=_('Refresh only the current task tab'),
-                  menu='view')
-        newaction(_("Load &All Revisions"), self.loadall,
-                  enabled='repoopen', menu='view', shortcut='Shift+Ctrl+A',
-                  tooltip=_('Load all revisions into graph'))
+        newnamed('Workbench.refresh', self.refresh, icon='view-refresh',
+                 enabled='repoopen', menu='view', toolbar='edit',
+                 tooltip=_('Refresh current repository'))
+        newnamed('Workbench.refreshTaskTabs', self._repofwd('reloadTaskTab'),
+                 enabled='repoopen',
+                 tooltip=_('Refresh only the current task tab'),
+                 menu='view')
+        newnamed('RepoView.loadAllRevisions', self.loadall,
+                 enabled='repoopen', menu='view',
+                 tooltip=_('Load all revisions into graph'))
 
-        self.actionAbort = \
-        newaction(_('Cancel'), self._abortCommands, icon='process-stop',
-                  toolbar='edit',
-                  tooltip=_('Stop current operation'))
+        self.actionAbort = newnamed('Workbench.abort', self._abortCommands,
+                                    icon='process-stop',
+                                    toolbar='edit',
+                                    tooltip=_('Stop current operation'))
         self.actionAbort.setEnabled(False)
 
         newseparator(toolbar='edit')
-        newaction(_("Go to current revision"), self._repofwd('gotoParent'),
-                  icon='go-home', tooltip=_('Go to current revision'),
-                  enabled='repoopen', toolbar='edit', shortcut='Ctrl+.')
-        newaction(_("&Goto Revision..."), self._gotorev, icon='go-to-rev',
-                  shortcut='Ctrl+/', enabled='repoopen',
-                  tooltip=_('Go to a specific revision'),
-                  menu='view', toolbar='edit')
+        newnamed('RepoView.goToWorkingParent', self._repofwd('gotoParent'),
+                 icon='go-home', tooltip=_('Go to current revision'),
+                 enabled='repoopen', toolbar='edit')
+        newnamed('RepoView.goToRevision', self._gotorev, icon='go-to-rev',
+                 tooltip=_('Go to a specific revision'),
+                 enabled='repoopen', menu='view', toolbar='edit')
 
-        self.actionBack = \
-        newaction(_("Back"), self._repofwd('back'), icon='go-previous',
-                  shortcut=QKeySequence.Back,
-                  enabled=False, toolbar='edit')
-        self.actionForward = \
-        newaction(_("Forward"), self._repofwd('forward'), icon='go-next',
-                  shortcut=QKeySequence.Forward,
-                  enabled=False, toolbar='edit')
+        self.actionBack = newnamed('RepoView.goBack', self._repofwd('back'),
+                                   icon='go-previous',
+                                   enabled=False, toolbar='edit')
+        self.actionForward = newnamed('RepoView.goForward',
+                                      self._repofwd('forward'), icon='go-next',
+                                      enabled=False, toolbar='edit')
         newseparator(toolbar='edit', menu='View')
 
-        self.filtertbaction = \
-        newaction(_('&Filter Toolbar'), self._repotogglefwd('toggleFilterBar'),
-                  icon='view-filter', shortcut='Ctrl+S', enabled='repoopen',
-                  toolbar='edit', menu='View', checkable=True,
-                  tooltip=_('Filter graph with revision sets or branches'))
+        self.filtertbaction = newnamed('RepoView.showFilterBar',
+                                       self._repotogglefwd('toggleFilterBar'),
+                                       icon='view-filter', enabled='repoopen',
+                                       toolbar='edit', menu='View',
+                                       checkable=True,
+                                       tooltip=_('Filter graph with revision '
+                                                 'sets or branches'))
 
         menu = QMenu(_('&Workbench Toolbars'), self)
         menu.addAction(self.edittbar.toggleViewAction())
@@ -340,80 +327,81 @@ class Workbench(QMainWindow):
 
         newseparator(toolbar='edit')
         menuSync = self.menuRepository.addMenu(_('S&ynchronize'))
-        a = newaction(_("&Lock File..."), self._repofwd('lockTool'),
-                    icon='thg-password', enabled='repoopen',
-                    menu='repository', toolbar='edit',
-                    tooltip=_('Lock or unlock files'))
+        a = newnamed('Repository.lockFile', self._repofwd('lockTool'),
+                     icon='thg-password', enabled='repoopen',
+                     menu='repository', toolbar='edit',
+                     tooltip=_('Lock or unlock files'))
         self.lockToolAction = a
         newseparator(menu='repository')
-        newaction(_("&Update..."), self._repofwd('updateToRevision'),
-                  icon='hg-update', enabled='repoopen',
-                  menu='repository', toolbar='edit',
-                  tooltip=_('Update working directory or switch revisions'))
-        newaction(_("&Shelve..."), self._repofwd('shelve'), icon='hg-shelve',
-                  enabled='repoopen', menu='repository')
-        newaction(_("&Import Patches..."), self._repofwd('thgimport'),
-                  icon='hg-import', enabled='repoopen', menu='repository')
-        newaction(_("U&nbundle..."), self._repofwd('unbundle'),
-                  icon='hg-unbundle', enabled='repoopen', menu='repository')
+        newnamed('Repository.update', self._repofwd('updateToRevision'),
+                 icon='hg-update', enabled='repoopen',
+                 menu='repository', toolbar='edit',
+                 tooltip=_('Update working directory or switch revisions'))
+        newnamed('Repository.shelve', self._repofwd('shelve'), icon='hg-shelve',
+                 enabled='repoopen', menu='repository')
+        newnamed('Repository.import', self._repofwd('thgimport'),
+                 icon='hg-import', enabled='repoopen', menu='repository')
+        newnamed('Repository.unbundle', self._repofwd('unbundle'),
+                 icon='hg-unbundle', enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_('&Merge...'), self._repofwd('mergeWithOtherHead'),
-                  icon='hg-merge', enabled='repoopen',
-                  menu='repository', toolbar='edit',
-                  tooltip=_('Merge with the other head of the current branch'))
-        newaction(_("&Resolve..."), self._repofwd('resolve'),
-                  enabled='repoopen', menu='repository')
+        newnamed('Repository.merge', self._repofwd('mergeWithOtherHead'),
+                 icon='hg-merge', enabled='repoopen',
+                 menu='repository', toolbar='edit',
+                 tooltip=_('Merge with the other head of the current branch'))
+        newnamed('Repository.resolve', self._repofwd('resolve'),
+                 enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_("R&ollback/Undo..."), self._repofwd('rollback'),
-                  shortcut='Ctrl+u',
-                  enabled='repoopen', menu='repository')
+        newnamed('Repository.rollback', self._repofwd('rollback'),
+                 enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_("&Purge..."), self._repofwd('purge'), enabled='repoopen',
-                  icon='hg-purge', menu='repository')
+        newnamed('Repository.purge', self._repofwd('purge'), enabled='repoopen',
+                 icon='hg-purge', menu='repository')
         newseparator(menu='repository')
-        newaction(_("&Bisect..."), self._repofwd('bisect'),
-                  enabled='repoopen', menu='repository')
+        newnamed('Repository.bisect', self._repofwd('bisect'),
+                 enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_("&Verify"), self._repofwd('verify'), enabled='repoopen',
-                  menu='repository')
-        newaction(_("Re&cover"), self._repofwd('recover'),
-                  enabled='repoopen', menu='repository')
+        newnamed('Repository.verify', self._repofwd('verify'),
+                 enabled='repoopen', menu='repository')
+        newnamed('Repository.recover', self._repofwd('recover'),
+                 enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_("E&xplore"), self.explore, shortcut='Shift+Ctrl+X',
-                  icon='system-file-manager', enabled='repoopen',
-                  menu='repository')
-        newaction(_("&Terminal"), self.terminal, shortcut='Shift+Ctrl+T',
-                  icon='utilities-terminal', enabled='repoopen',
-                  menu='repository')
-        newaction(_("&Web Server"), self.serve, menu='repository',
-                  icon='hg-serve')
+        newnamed('Workbench.openFileManager', self.explore,
+                 icon='system-file-manager', enabled='repoopen',
+                 menu='repository')
+        newnamed('Workbench.openTerminal', self.terminal,
+                 icon='utilities-terminal', enabled='repoopen',
+                 menu='repository')
+        newnamed('Workbench.webServer', self.serve, menu='repository',
+                 icon='hg-serve')
 
-        newaction(_("&Help"), self.onHelp, menu='help', icon='help-browser')
-        newaction(_("E&xplorer Help"), self.onHelpExplorer, menu='help')
+        newnamed('Workbench.help', self.onHelp, menu='help',
+                 icon='help-browser')
+        newnamed('Workbench.explorerHelp', self.onHelpExplorer, menu='help')
         visiblereadme = 'repoopen'
         if self._config.configString('tortoisehg', 'readme'):
             visiblereadme = True
-        newaction(_("&Readme"), self.onReadme, menu='help', icon='help-readme',
-                  visible=visiblereadme, shortcut='Ctrl+F1')
+        newnamed('Workbench.openReadme', self.onReadme, menu='help',
+                 icon='help-readme', visible=visiblereadme)
         newseparator(menu='help')
-        newaction(_("About &Qt"), QApplication.aboutQt, menu='help')
-        newaction(_("&About TortoiseHg"), self.onAbout, menu='help', icon='thg')
+        newnamed('Workbench.aboutQt', QApplication.aboutQt, menu='help')
+        newnamed('Workbench.about', self.onAbout, menu='help', icon='thg')
 
-        newaction(_('&Incoming'), data='incoming', icon='hg-incoming',
-                  enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+,')
-        newaction(_('&Pull'), data='pull', icon='hg-pull',
-                  enabled='repoopen', toolbar='sync')
-        newaction(_('&Outgoing'), data='outgoing', icon='hg-outgoing',
-                  enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+.')
-        newaction(_('P&ush'), data='push', icon='hg-push',
-                  enabled='repoopen', toolbar='sync')
+        newnamed('Repository.incoming', data='incoming', icon='hg-incoming',
+                 enabled='repoopen', toolbar='sync')
+        newnamed('Repository.pull', data='pull', icon='hg-pull',
+                 enabled='repoopen', toolbar='sync')
+        newnamed('Repository.outgoing', data='outgoing', icon='hg-outgoing',
+                 enabled='repoopen', toolbar='sync')
+        newnamed('Repository.push', data='push', icon='hg-push',
+                 enabled='repoopen', toolbar='sync')
         menuSync.addActions(self.synctbar.actions())
         menuSync.addSeparator()
 
-        action = QAction(_('&Sync Bookmarks...'), self)
+        action = QAction(self)
         action.setIcon(qtlib.geticon('thg-sync-bookmarks'))
         self._actionavails['repoopen'].append(action)
         action.triggered.connect(self._runSyncBookmarks)
+        self._actionregistry.registerAction('Repository.syncBookmarks', action)
         menuSync.addAction(action)
 
         self._lastRepoSyncPath = {}
@@ -625,6 +613,16 @@ class Workbench(QMainWindow):
             getattr(self, '%stbar' % toolbar).addAction(action)
         return action
 
+    def _addNewNamedAction(self, name, slot=None, icon=None, checkable=False,
+                           tooltip=None, data=None, enabled=None,
+                           visible=None, menu=None, toolbar=None):
+        """Create new action and register it as user-configurable"""
+        a = self._addNewAction('', slot=slot, icon=icon, checkable=checkable,
+                               tooltip=tooltip, data=data, enabled=enabled,
+                               visible=visible, menu=menu, toolbar=toolbar)
+        self._actionregistry.registerAction(name, a)
+        return a
+
     def _addNewSeparator(self, menu=None, toolbar=None):
         """Insert a separator action; returns nothing"""
         if menu:
@@ -1190,3 +1188,9 @@ class Workbench(QMainWindow):
     @pyqtSlot()
     def _editCustomToolsSettings(self):
         self.editSettings('tools')
+
+    @pyqtSlot()
+    def _openShortcutSettingsDialog(self):
+        dlg = shortcutsettings.ShortcutSettingsDialog(
+            self._actionregistry, self)
+        dlg.exec_()
