@@ -18,6 +18,7 @@ import time
 from hgext import mq as mqmod
 from mercurial import (
     cmdutil,
+    config,
     dispatch as dispatchmod,
     encoding,
     error,
@@ -26,7 +27,7 @@ from mercurial import (
     filemerge,
     filesetlang,
     mdiff,
-    merge as mergemod,
+    mergestate as mergestatemod,
     patch as patchmod,
     pathutil,
     pycompat,
@@ -35,6 +36,7 @@ from mercurial import (
     revsetlang,
     scmutil,
     state as statemod,
+    subrepo,
     subrepoutil,
     ui as uimod,
     util,
@@ -70,6 +72,13 @@ if TYPE_CHECKING:
         Union,
         overload,
     )
+    from mercurial import (
+        context,
+        localrepo,
+    )
+    from .typelib import (
+        IniConfig,
+    )
 
 nullsubrepostate = subrepoutil.nullstate
 _encoding = pycompat.sysstr(encoding.encoding)
@@ -202,7 +211,7 @@ def fromutf(s):
         return fromunicode(s.decode('utf-8'), 'replace')
     except UnicodeDecodeError:
         # can't round-trip
-        return str(fromunicode(s.decode('utf-8', 'replace'), 'replace'))
+        return bytes(fromunicode(s.decode('utf-8', 'replace'), 'replace'))
 
 
 getcwdb = encoding.getcwd
@@ -249,6 +258,7 @@ def repoidnode(repo):
     return _firstchangectx(repo).node()
 
 def _getfirstrevisionlabel(repo, ctx):
+    # type: (...) -> Optional[bytes]
     # see context.changectx for look-up order of labels
 
     bookmarks = ctx.bookmarks()
@@ -270,7 +280,7 @@ def _getfirstrevisionlabel(repo, ctx):
         return branch
 
 def getrevisionlabel(repo, rev):
-    # type: (Any, Optional[int]) -> Optional[bytes]
+    # type: (Any, Optional[int]) -> Optional[Text]
     """Return symbolic name for the specified revision or stringfy it"""
     if rev is None:
         return None  # no symbol for working revision
@@ -278,9 +288,9 @@ def getrevisionlabel(repo, rev):
     ctx = repo[rev]
     label = _getfirstrevisionlabel(repo, ctx)
     if label and ctx == scmutil.revsymbol(repo, label):
-        return label
+        return tounicode(label)
 
-    return b'%d' % rev
+    return '%d' % rev
 
 def getmqpatchtags(repo):
     # type: (...) -> List[bytes]
@@ -301,11 +311,11 @@ def getcurrentqqueue(repo):
         cur = cur[8:]
     return cur
 
-def gitcommit(ctx):
-    # type: (...) -> Optional[bytes]
+def gitcommit_full(ctx):
+    # type: (...) -> Optional[Text]
     """
     If the hggit extension is loaded, and the repository is a git clone,
-    returns the git commit hash of the current revision
+    returns the complete git commit hash of the current revision
     """
 
     repo = ctx._repo
@@ -313,11 +323,21 @@ def gitcommit(ctx):
     if b'hggit' not in repo.extensions():
         return None
 
-    fullgitnode = repo.githandler.map_git_get(ctx.hex())
+    fullgitnode = tounicode(repo.githandler.map_git_get(ctx.hex()))
     if fullgitnode is None:
         return None
 
-    return fullgitnode[:12]
+    return fullgitnode
+
+def gitcommit_short(ctx):
+    # type: (...) -> Optional[Text]
+    """
+    If the hggit extension is loaded, and the repository is a git clone,
+    returns the short (12 digits) git commit hash of the current revision
+    """
+
+    fullgitnode = gitcommit_full(ctx)
+    return None if fullgitnode is None else fullgitnode[:12]
 
 def getqqueues(repo):
     # type: (...) -> List[Text]
@@ -341,12 +361,8 @@ def readgraftstate(repo):
 
     return None
 
-try:
-    from mercurial import mergestate as mergestatemod
-    mergestate = mergestatemod.mergestate
-except (AttributeError, ImportError):
-    # hg<5.5 b7808443ed6a
-    mergestate = mergemod.mergestate
+
+mergestate = mergestatemod.mergestate
 readmergestate = mergestate.read
 
 def readundodesc(repo):
@@ -398,7 +414,19 @@ def allextensions():
             exts['hgsubversion'] = _('hgsubversion packaged with thg')
         if 'hggit' not in exts:
             exts['hggit'] = _('hggit packaged with thg')
-        exts.pop('mercurial_extension_utils', None)  # Part of keyring extension
+        if 'evolve' not in exts:
+            exts['evolve'] = _('evolve packaged with thg')
+        if 'topic' not in exts:
+            exts['topic'] = _('topic packaged with thg')
+
+    exts.pop('mercurial_extension_utils', None)  # Part of keyring extension
+    exts.pop('hooklib', None)  # Internal, but also needs [hooks] setup
+
+    # The core git extension requires pygit2, which doesn't support py2, so no
+    # sense in advertising it.
+    if not pycompat.ispy3:
+        exts.pop('git', None)
+
     return exts
 
 def validateextensions(enabledexts):
@@ -442,6 +470,19 @@ def loadextensions(ui):
     _wrapextensionsloader()  # enable blacklist of extensions
     extensions.loadall(ui)
 
+
+def hastopicext(repo):
+    # type: (localrepo.localrepository) -> bool
+    """Indicate is the topic extension is enabled for ``repo``."""
+    hastopicext = False
+
+    try:
+        hastopicext = extensions.find(b'topic').hastopicext(repo)
+    except (KeyError, AttributeError):
+        pass
+
+    return hastopicext
+
 # TODO: provide singular canonpath() wrapper instead?
 def canonpaths(list):
     # type: (Iterable[bytes]) -> List[bytes]
@@ -466,6 +507,14 @@ def canonpaths(list):
                 canonpats.append(f)
     return canonpats
 
+
+def normpath(path):
+    # type: (Text) -> Text
+    """Normalize a path in the same manner as mercurial.util.normpath()."""
+    # TODO: Do a unicode port of util.normpath() here.
+    return tounicode(util.normpath(fromunicode(path)))
+
+
 def normreporoot(path):
     # type: (Text) -> Text
     """Normalize repo root path in the same manner as localrepository"""
@@ -562,7 +611,7 @@ tortoisehgtoollocations = (
 )
 
 def tortoisehgtools(uiorconfig, selectedlocation=None):
-    # TODO: type annotation
+    # type: (IniConfig, Optional[Text]) -> Tuple[Dict[Text, Dict[Text, Union[Text, bool]]], List[Text]]
     """Parse 'tortoisehg-tools' section of ini file.
 
     >>> from pprint import pprint
@@ -673,17 +722,22 @@ def tortoisehgtools(uiorconfig, selectedlocation=None):
         configlist = uiorconfig.configlist
     else:
         configitems = uiorconfig.items
+        assert not isinstance(uiorconfig, uimod.ui)  # help pytype
+
         def configlist(section, name):
             return uiorconfig.get(section, name, b'').split()
 
-    tools = {}
+    tools = {}  # type: Dict[Text, Dict[Text, Union[Text, bool]]]
     for key, value in configitems(b'tortoisehg-tools'):
         toolname, field = tounicode(key).split('.', 1)
         if toolname not in tools:
             tools[toolname] = {}
-        bvalue = stringutil.parsebool(value)
-        if bvalue is not None:
-            value = bvalue
+        if field == 'showoutput':
+            bvalue = stringutil.parsebool(value)
+            if bvalue is not None:
+                value = bvalue
+            else:
+                value = True
         else:
             value = tounicode(value)
         tools[toolname][field] = value
@@ -697,8 +751,8 @@ def tortoisehgtools(uiorconfig, selectedlocation=None):
 
     guidef = configlist(b'tortoisehg',
                         pycompat.sysbytes(selectedlocation)) or []
-    toollist = []
-    selectedtools = {}
+    toollist = []  # type: List[Text]
+    selectedtools = {}  # type: Dict[Text, Dict[Text, Union[Text, bool]]]
     for name in guidef:
         name = tounicode(name)
         if name != '|':
@@ -899,7 +953,7 @@ def longsummary(description, limit=None):
     return summary
 
 def getDeepestSubrepoContainingFile(wfile, ctx):
-    # TODO: type annotation
+    # type: (bytes, context.changectx) -> Tuple[Optional[bytes], bytes, context.changectx]
     """
     Given a filename and context, get the deepest subrepo that contains the file
 
@@ -907,18 +961,23 @@ def getDeepestSubrepoContainingFile(wfile, ctx):
     its containing subrepo
     """
     if wfile in ctx:
-        return '', wfile, ctx
-    for wsub in ctx.substate:
+        return b'', wfile, ctx
+
+    # TODO: figure out how to annotate ctx.substate so that pytype sees the
+    #   correct type instead of Any.
+    for wsub in ctx.substate:  # type: bytes
         if wfile.startswith(wsub):
-            srev = ctx.substate[wsub][1]
-            stype = ctx.substate[wsub][2]
-            if stype != 'hg':
+            srev = ctx.substate[wsub][1]  # type: bytes
+            stype = ctx.substate[wsub][2]  # type: bytes
+            if stype != b'hg':
                 continue
             if not os.path.exists(ctx._repo.wjoin(wsub)):
                 # Maybe the repository does not exist in the working copy?
                 continue
             try:
-                sctx = ctx.sub(wsub)._repo[srev]
+                s = ctx.sub(wsub)
+                assert isinstance(s, subrepo.hgsubrepo)  # help pytype
+                sctx = s._repo[srev]  # type: context.changectx
             except:
                 # The selected revision does not exist in the working copy
                 continue
